using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib6;

namespace XNA_Collisions
{
	public enum GraphicsObjectType
	{
		gotPoint,
		gotLine,
		gotCircle,
	};

	public class Model
	{
		private UWB_Primitive mCurrentObject;

		private UWB_PrimitiveCircle mCircle;
		private UWB_PrimitiveLine mLine;
		private UWB_PrimitivePoint mPoint;

		private UWB_Clock mStopWatch;
		private UWB_BoundingBox mWorldBounds;
		private bool mShowBoundingVolumes;

		private UWB_XNADrawHelper mDrawHelper;

		public Model()
		{
			mWorldBounds = new UWB_BoundingBox();

			mDrawHelper = new UWB_XNADrawHelper();

			mCircle = new UWB_PrimitiveCircle();
			mLine = new UWB_PrimitiveLine();
			mPoint = new UWB_PrimitivePoint();

			setCurrentTypeToPoint();

			mStopWatch = new UWB_Clock();
		}

		public void setWorldSize(float width, float height)
		{
			mWorldBounds.setCorners(Vector3.Zero, new Vector3(width, height, 0));
		}

		public void getWorldSize(ref float width, ref float height)
		{
			width = mWorldBounds.width();
			height = mWorldBounds.height();
		}

		public void setCurrentTypeToPoint()
		{
			mCurrentObject = mPoint;
		}

		public void setCurrentTypeToLine()
		{
			mCurrentObject = mLine;
		}

		public void setCurrentTypeToCircle()
		{
			mCurrentObject = mCircle;
		}

		public void setObjectColor(Color color)
		{
			mCurrentObject.setFlatColor(color);
		}

		public void setShadingColor(Color color)
		{
			mCurrentObject.setShadingColor(color);
		}

		public void setShadeMode(eShadeMode shadeMode)
		{
			mCurrentObject.setShadeMode(shadeMode);
		}

		public void setPointSize(float size)
		{
			mCurrentObject.setPointSize(size);
		}

		public void setFillMode(eFillMode fillMode)
		{
			mCurrentObject.setFillMode(fillMode);
		}

		public eShadeMode getShadeMode()
		{
			return mCurrentObject.getShadeMode();
		}

		public eFillMode getFillMode()
		{
			return mCurrentObject.getFillMode();
		}

		public float getPointSize()
		{
			return mCurrentObject.getPointSize();
		}

		public void startDefineGraphicsObject(float worldX, float worldY)
		{
			mCurrentObject.mouseDownVertex(0, worldX, worldY);
			mCurrentObject.setVelocity(Vector3.Zero);
		}

		public void endDefineGraphicsObject(float worldX, float worldY)
		{
			mCurrentObject.mouseDownVertex(1, worldX, worldY);
		}

		public void moveGraphicsObject(float worldX, float worldY)
		{
			mCurrentObject.moveTo(worldX, worldY);
		}

		public void updateSimulation()
		{
			Vector3 ptLoc = mPoint.getLocation();
			if(!mWorldBounds.containsPoint(ptLoc))
				mPoint.setVelocity(-mPoint.getVelocity());

			Vector3 cirLoc = mCircle.getLocation();
			if (!mWorldBounds.containsPoint(cirLoc))
				mCircle.setVelocity(-mCircle.getVelocity());

			Vector3 lnLoc = mLine.getLocation();
			if (!mWorldBounds.containsPoint(lnLoc))
				mLine.setVelocity(-mLine.getVelocity());

			eLevelofDetail lod = mDrawHelper.getLod();
			UWB_BoundingVolume circleBV = mCircle.getBoundingVolume(lod);
			UWB_BoundingVolume pointBV = mPoint.getBoundingVolume(lod);
			UWB_BoundingVolume lineBV = mLine.getBoundingVolume(lod);

			bool circleCollision = false;
			bool pointCollision = false;
			bool lineCollision = false;

			if (circleBV != null 
				&& pointBV != null
				&& circleBV.intersects(pointBV))
			{
				//Collision Response
				if (!circleCollision)
					mCircle.collisionResponse(mPoint, ptLoc);
				if (!pointCollision)
					mPoint.collisionResponse(mCircle, cirLoc);
				circleCollision = true;
				pointCollision = true;
			}

			if (circleBV != null
				&& lineBV != null
				&& circleBV.intersects(lineBV))
			{
				//Collision Response
				if (!circleCollision)
					mCircle.collisionResponse(mLine, lnLoc);
				if (!lineCollision)
					mLine.collisionResponse(mCircle, cirLoc);
				circleCollision = true;
				lineCollision = true;
			}

			if (pointBV != null
				&& lineBV != null
				&& pointBV.intersects(lineBV))
			{
				//Collision Response
				if (!pointCollision)
					mPoint.collisionResponse(mLine, lnLoc);
				if (!lineCollision)
					mLine.collisionResponse(mPoint, ptLoc);
				pointCollision = true;
				lineCollision = true;
			}

			float elapsedSeconds = mStopWatch.getSecondsElapsed();
			mPoint.update(elapsedSeconds);
			mLine.update(elapsedSeconds);
			mCircle.update(elapsedSeconds);
		}

		public void drawModel()
		{
			eLevelofDetail lod = mDrawHelper.getLod();
			mCircle.draw(lod, mDrawHelper);
			mLine.draw(lod, mDrawHelper);
			mPoint.draw(lod, mDrawHelper);

			if (mShowBoundingVolumes)
			{
				Color color = new Color(255, 0, 0);
				mCircle.drawBoundingVolume(lod, mDrawHelper, color);
				mLine.drawBoundingVolume(lod, mDrawHelper, color);
				mPoint.drawBoundingVolume(lod, mDrawHelper, color);
			}
		}

		public void placeGraphicsObjectInMotion()
		{
			float x = UWB_Utility.UWB_RandomNumber(-30, 30);
			float y = UWB_Utility.UWB_RandomNumber(-30, 30);
			Vector3 randomVelocity = new Vector3(x, y, 0);

			mCurrentObject.setVelocity(randomVelocity);
		}

		public void setLevelOfDetail(eLevelofDetail lod)
		{
			mDrawHelper.setLod(lod);
		}

		public eLevelofDetail getLevelOfDetail()
		{
			return mDrawHelper.getLod();
		}

		public void showBoundingVolumes(bool on)
		{
			mShowBoundingVolumes = on;
		}
	}
}

